LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;
USE work.controlFuncs.ALL;

ENTITY controller IS
  
  GENERIC (
    nPlayers : INTEGER := 32);          -- Number of Players

  PORT (
    -- Input
    clock       : IN  STD_LOGIC;
    enable      : IN  STD_LOGIC;
    reset       : IN  STD_LOGIC;
    bistEnable  : IN  STD_LOGIC;
    -- Input from Player
    pHit        : IN  STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
    pStand      : IN  STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
    -- Input from Card Generator
    issued      : IN  STD_LOGIC;
    -- Input from Bank
    gameOver    : IN  STD_LOGIC;
    balanceRdy  : IN  STD_LOGIC;
    -- Input from Score Keeper
    skBust      : IN  STD_LOGIC;
    -- Output
    playerID    : OUT STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0);
    globalReset : OUT STD_LOGIC;
    -- Output to Card Generator
    cgIssue     : OUT STD_LOGIC;
    cgNewHand   : OUT STD_LOGIC;
    cgBist      : OUT STD_LOGIC;
    -- Output to Score Keeper
    handOver    : OUT STD_LOGIC;
    pbr         : out STD_LOGIC);



END controller;

ARCHITECTURE mixed OF controller IS

  SIGNAL state    : STD_LOGIC_VECTOR(3 DOWNTO 0) := "0000";
  SIGNAL numCards : STD_LOGIC                    := '0';
  signal ipbr : STD_LOGIC := '0';

BEGIN

  main : PROCESS (clock, reset, issued)
    VARIABLE initialPlayer : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0) := (OTHERS => '0');
    VARIABLE controlID     : STD_LOGIC_VECTOR(nPlayers-1 DOWNTO 0) := (OTHERS => '0');
    VARIABLE timeout       : STD_LOGIC_VECTOR(4 DOWNTO 0);
  BEGIN  -- process main
    IF reset = '1' THEN                 -- asynchronous reset (active low)
      initialPlayer := (OTHERS => '0');
      initialPlayer(initialPlayer'right) := '1';
      playerID <= controlID;
      globalReset <= '1';
      state <= "0000";
      cgBist <= '0';
      handOver <= '0';
      ipbr <= '0';
      pbr <= '0';
    ELSIF clock'event AND clock = '1' AND reset = '0' THEN  -- rising clock edge
      globalReset <= '0'; 
      cgBist <= '0';
      CASE state IS
        WHEN "0000" =>                  -- New Hand Init
          cgNewHand <= '1';

          playerID  <= initialPlayer;
          controlID := initialPlayer;
          state     <= "0001";
          --ASSERT state = "0001" REPORT "Failed to change state!!" SEVERITY note;
          timeout   := (OTHERS => '0');
        WHEN "0001" =>                  -- Bet mode
          cgNewHand <= '0';
          numCards  <= '0';

          -- Cycle Through Players
          -- If pStand or timeout
          
          IF (pStand AND controlID) = controlID OR timeout(timeout'left) = '1' THEN
            --ASSERT false REPORT "Going To Next Player" SEVERITY note;
            controlID := nextPlayerID(controlID);
            playerID  <= controlID;
            --ASSERT false REPORT "PlayerID Set" SEVERITY note;            
            timeout   := (OTHERS => '0');
            IF controlID = initialPlayer THEN
              state <= "0010";
            END IF;
          ELSE
            --   count timeout;
            timeout := timeout + 1;
            ASSERT timeout(timeout'left) = '0'
			 REPORT "Time Out!!! on player " & integer'image(conv_integer(controlID))  SEVERITY note;            

          END IF;
          
        WHEN "0010" =>                  -- Deal Mode

          --  Cycle Through Players
          --  For each player - dealer
          cgIssue <= '1';

          IF issued = '1' THEN
            cgIssue   <= '0';
            controlID := nextPlayerID(controlID);
            IF controlID(controlID'left) = '1' THEN
              controlID := nextPlayerID(controlID);
            END IF;
            playerID <= controlID;
            IF controlID = initialPlayer THEN
              timeout := (OTHERS => '0');
              IF numCards = '1' THEN
                state   <= "0011";
              ELSE
                numCards <= '1';
              END IF;

            END IF;
          END IF;
          
          
          
        WHEN "0011" =>                         -- Game Interaction
          -- For each player
          -- When player is enabled their hit/stand must be preset (i.e. 0's if
          -- they don't want to do anything yet). 
          IF skBust = '1' OR ((pStand AND controlID) = controlID) OR timeout(timeout'left) = '1' THEN
            timeout   := (OTHERS => '0');
            cgIssue   <= '0';
            controlID := nextPlayerID(controlID);
            IF controlID(controlID'left) = '1' THEN
              controlID := nextPlayerID(controlID);
            END IF;
            playerID <= controlID;
            IF controlID = initialPlayer THEN  -- service dealer
              controlID                 := (OTHERS => '0');
              controlID(controlID'left) := '1';
              playerID                  <= controlID;
              state                     <= "0110";
            END IF;
          ELSE
            timeout := timeout + 1;
            -- !!!!!! Does this work ? !!!!!
            -- Player must set pHit to zero after they receive a card.
            IF (pHit AND controlID) = controlID THEN
              cgIssue <= '1';
            ELSE
              cgIssue <= '0';
            END IF;

            IF issued = '1' THEN
              timeout := (OTHERS => '0');
            END IF;

          END IF;
          
          
        WHEN "0110" =>
          -- deal with dealer
          IF skBust = '1' OR ((pStand AND controlID) = controlID) OR timeout(timeout'left) = '1' THEN
            timeout   := (OTHERS => '0');
            cgIssue   <= '0';
            controlID := initialPlayer;
            playerID  <= controlID;
            handOver <= '1';
            state     <= "0100";

          ELSE
            timeout := timeout + 1;
            -- !!!!!! Does this work ? !!!!!
            -- Player must set pHit to zero after they receive a card.
            IF (pHit AND controlID) = controlID THEN
              cgIssue <= '1';
            ELSE
              cgIssue <= '0';
            END IF;

            IF issued = '1' THEN
              timeout := (OTHERS => '0');
            END IF;

          END IF;
-------------------------------------------------------------------------------
--        commented by JDW
--        when "0111" =>
--          if balanceRdy = '1' then
--            state <= "0100";
--          end if;
-------------------------------------------------------------------------------
        WHEN "0100" =>                  -- Results
          -- Cycle through each player
          IF balanceRdy = '1' THEN
            if ipbr = '1' then
              
              controlID := nextPlayerID(controlID);
              playerID  <= controlID;
              pbr <= '0';
              ipbr <= '0';

              IF controlID = initialPlayer THEN
                handOver <= '0';
                playerID <= (OTHERS => '0');

                state <= "0101";
              END IF;
            else
              pbr <= '1';
              ipbr <= '1';
            end if;
          END IF;

        WHEN "0101" =>                  -- Game Over

          IF gameOver = '1' THEN
            -- display results from banker
            ELSE
              state <= "0000";
          END IF;
        WHEN OTHERS => NULL;
      END CASE;
      
    ELSE
      globalReset <= '0'; 
    END IF;
  END PROCESS main;

END mixed;


